/**
 * Copyright (C) 2001-2005 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Authors: S.Chassande-Barrioz.
 * Created on 13 juin 2005
 *
 */
package org.objectweb.jorm.type.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.jorm.type.api.PTypeSpace;

/**
 * This class is an helper for the PType uses. The aim is to centralize the 
 * function around PType concept.
 *
 * @author S.Chassande-Barrioz
 */
public class TypeHelper {

    public final static TypeHelper instance = new TypeHelper();
    
    public final String getGenPaMethodPart(final PType pt) {
        switch (pt.getTypeCode()) {
        case PType.TYPECODE_BYTEARRAY:
            return "ByteArray";
		case PType.TYPECODE_REFERENCE:
			return "Ref";
		default:
		    return pt.getCodingName();
        }
    }
    
    public final String getPNameCodingMethodPart(PType pt) {
		if (pt.getTypeCode() == PType.TYPECODE_BYTEARRAY) {
		    return "";
		} else {
		    return pt.getCodingName();
		}
        
    }
    /**
     * Retrieves a value under an Object format. 
     * @param strValue is the String representation of the value
     * @param valueType is the type of the value
     * @return a string representation of the value in Object
     * example: 
     * getValueAsObjectForGen("1", PTypeSpace.INT) ==> "new Integer(1)"
     * getValueAsObjectForGen("myInteger", PTypeSpace.OBJINT) ==> "myInteger"
     */
	public final String getValueAsObjectForGen(final String strValue, 
	        final PType valueType) {
	    if (isObjectType(valueType)) {
	        return strValue;
	    } else {
			return "new " + toJavaLangPrimitive(valueType).getJavaName() 
				+ "(" + strValue + ")";
	    }
	}

	/**
	 * Retrieves a String representation of the default null value for a given
	 * PType.
	 */
	public final String getDefaultNullValueAsString(final PType type) {
		switch (type.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
			return "false";
		case PType.TYPECODE_BYTE:
			return "(byte) -1";
		case PType.TYPECODE_CHAR:
			return "(char) 0";
		case PType.TYPECODE_SHORT:
			return "(short) -1";
		case PType.TYPECODE_INT:
			return "(int) -1";
		case PType.TYPECODE_LONG:
			return "(long) -1";
		case PType.TYPECODE_FLOAT:
			return "(float) -1";
		case PType.TYPECODE_DOUBLE:
			return "(double) -1";
		default:
			return "null";
		}
	}
	
	/**
	 * Converts a PType representing java primitive type, in a PType 
	 * representing the corresponding java.lang.XXX type. If the specified does
	 * not correpond to a java primitive, the result is the specified 
	 * parameter (unchanged).
	 * @see #toPrimitive(PType)  
	 */
	public final PType toJavaLangPrimitive(final PType pt) {
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN:
			return PTypeSpace.OBJBOOLEAN;
		case PType.TYPECODE_CHAR:
			return PTypeSpace.OBJCHAR;
		case PType.TYPECODE_BYTE:
			return PTypeSpace.OBJBYTE;
		case PType.TYPECODE_SHORT:
			return PTypeSpace.OBJSHORT;
		case PType.TYPECODE_INT:
			return PTypeSpace.OBJINT;
		case PType.TYPECODE_LONG:
			return PTypeSpace.OBJLONG;
		case PType.TYPECODE_FLOAT:
			return PTypeSpace.OBJFLOAT;
		case PType.TYPECODE_DOUBLE:
			return PTypeSpace.OBJDOUBLE;
		default:
			return pt;
		}
	}

	/**
	 * Converts a PType representing java.lang.XXX type in a PType representing
	 * the corresponding java primitive type. If the specified does not 
	 * correpond to a java.lang.XXX, the result is the specified 
	 * parameter (unchanged).
	 * @see #toJavaLangPrimitive(PType)  
	 */
	public final PType toPrimitive(final PType pt) {
		switch (pt.getTypeCode()) {
		case PType.TYPECODE_OBJBOOLEAN:
			return PTypeSpace.BOOLEAN;
		case PType.TYPECODE_OBJCHAR:
			return PTypeSpace.CHAR;
		case PType.TYPECODE_OBJBYTE:
			return PTypeSpace.BYTE;
		case PType.TYPECODE_OBJSHORT:
			return PTypeSpace.SHORT;
		case PType.TYPECODE_OBJINT:
			return PTypeSpace.INT;
		case PType.TYPECODE_OBJLONG:
			return PTypeSpace.LONG;
		case PType.TYPECODE_OBJFLOAT:
			return PTypeSpace.FLOAT;
		case PType.TYPECODE_OBJDOUBLE:
			return PTypeSpace.DOUBLE;
		default:
			return pt;
		}
	}
	
	/**
	 * Indicates is a given PType is an Object type (true value) or a java
	 * primivite type such as long, int, ... (false value). Of course if
	 * the given is an Object type that means null is a possible value 
	 * for this type.
	 */
	public final boolean isObjectType(final PType pt) {
	    short typeCode = (short) pt.getTypeCode();
		switch(typeCode) {
		case PType.TYPECODE_BOOLEAN:
		case PType.TYPECODE_BYTE:
		case PType.TYPECODE_CHAR:
		case PType.TYPECODE_SHORT:
		case PType.TYPECODE_INT:
		case PType.TYPECODE_LONG:
		case PType.TYPECODE_FLOAT:
		case PType.TYPECODE_DOUBLE:
			return false;
		default:
			return true;
		}
	}

	public final static String CODER_ITF = "PNameCoder";
	
	/**
	 * Retrieves the String representing the coding type corresponding to
	 * a given PType.
	 * example:
	 * getCodingTypeAsString(PType.TYPECODE_SHORT) ==> "PNameCoder.CTSHORT"
	 */
	public final String getCodingTypeAsString(final int typecode) {
		switch (typecode) {
		case PType.TYPECODE_CHAR:
			return CODER_ITF + ".CTCHAR";
		case PType.TYPECODE_OBJCHAR:
			return CODER_ITF + ".CTOCHAR";

		case PType.TYPECODE_BYTE:
			return CODER_ITF + ".CTBYTE";
		case PType.TYPECODE_OBJBYTE:
			return CODER_ITF + ".CTOBYTE";

		case PType.TYPECODE_SHORT:
			return CODER_ITF + ".CTSHORT";
		case PType.TYPECODE_OBJSHORT:
			return CODER_ITF + ".CTOSHORT";

		case PType.TYPECODE_INT:
			return CODER_ITF + ".CTINT";
		case PType.TYPECODE_OBJINT:
			return CODER_ITF + ".CTOINT";

		case PType.TYPECODE_LONG:
			return CODER_ITF + ".CTLONG";
		case PType.TYPECODE_OBJLONG:
			return CODER_ITF + ".CTOLONG";

		case PType.TYPECODE_STRING:
			return CODER_ITF + ".CTSTRING";

		case PType.TYPECODE_BYTEARRAY:
			return CODER_ITF + ".CTBYTEARRAY";
		case PType.TYPECODE_CHARARRAY:
			return CODER_ITF + ".CTCHARARRAY";
		case PType.TYPECODE_DATE:
			return CODER_ITF + ".CTDATE";
		case PType.TYPECODE_BIGDECIMAL:
			return CODER_ITF + ".CTBIGDECIMAL";
		case PType.TYPECODE_BIGINTEGER:
			return CODER_ITF + ".CTBIGINTEGER";
		}
		return null;
	}

	public final String getTupleGetterAsString(PType type) {
		switch (type.getTypeCode()) {
		case PType.TYPECODE_BOOLEAN: return "getBoolean";
		case PType.TYPECODE_BYTE: return "getByte";
		case PType.TYPECODE_CHAR: return "getChar";
		case PType.TYPECODE_DATE: return "getDate";
		case PType.TYPECODE_CHARARRAY: return "getCharArray";
		case PType.TYPECODE_BYTEARRAY: return "getByteArray";
		case PType.TYPECODE_SHORT: return "getShort";
		case PType.TYPECODE_INT: return "getInt";
		case PType.TYPECODE_LONG: return "getLong";
		case PType.TYPECODE_FLOAT: return "getFloat";
		case PType.TYPECODE_DOUBLE: return "getDouble";
		case PType.TYPECODE_STRING: return "getString";
		case PType.TYPECODE_BIGDECIMAL: return "getBigDecimal";
		case PType.TYPECODE_BIGINTEGER: return "getBigInteger";

		case PType.TYPECODE_SERIALIZED: 
		default:
		    return "getObject";
		}
	    
	}
}
