/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 * 
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre
 */

package org.objectweb.medor.expression.lib;

import org.objectweb.jorm.type.api.PType;
import org.objectweb.medor.expression.api.BinaryArithmeticOperator;
import org.objectweb.medor.expression.api.ExpressionException;
import org.objectweb.medor.expression.api.MalformedExpressionException;
import org.objectweb.medor.expression.api.Operand;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.expression.api.TypingException;
import org.objectweb.medor.expression.api.Expression;
import org.objectweb.medor.expression.type.ExpressionTypeHelper;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;

/**
 *
 * @author Sebastien Chassande-Barrioz
 */
public abstract class BasicBinaryArithmeticOperator
        extends BasicBinaryOperator
        implements BinaryArithmeticOperator {
	
	public BasicBinaryArithmeticOperator() {
		super();
	}

	public BasicBinaryArithmeticOperator(Expression e0, Expression e1) {
		super(e0, e1);
	}

	public BasicBinaryArithmeticOperator(PType type) {
		super(type);
	}

	public BasicBinaryArithmeticOperator(PType type, Expression e0, Expression e1) {
		super(type, e0, e1);
	}
    // IMPLEMENTATION OF THE Expression INTERFACE //
    //--------------------------------------------//

    public Operand evaluate(ParameterOperand[] pos, Object o)
            throws ExpressionException {
        try {
            expressions[1].evaluate(pos, o);
            switch (type.getTypeCode()) {
                case PType.TYPECODE_BYTE:
                case PType.TYPECODE_OBJBYTE:
                    result.setValue(evaluate(expressions[0].evaluate(pos, o).getByte(),
									expressions[1].evaluate(pos, o).getByte()));
                    break;
                case PType.TYPECODE_CHAR:
                case PType.TYPECODE_OBJCHAR:
                    result.setValue(evaluate(expressions[0].evaluate(pos, o).getChar(),
									expressions[1].evaluate(pos, o).getChar()));
                    break;
                case PType.TYPECODE_SHORT:
                case PType.TYPECODE_OBJSHORT:
                    result.setValue(evaluate(expressions[0].evaluate(pos, o).getShort(),
									expressions[1].evaluate(pos, o).getShort()));
                    break;
                case PType.TYPECODE_INT:
                case PType.TYPECODE_OBJINT:
                    result.setValue(evaluate(expressions[0].evaluate(pos, o).getInt(),
									expressions[1].evaluate(pos, o).getInt()));
                    break;
                case PType.TYPECODE_LONG:
                case PType.TYPECODE_OBJLONG:
                    result.setValue(evaluate(expressions[0].evaluate(pos, o).getLong(),
									expressions[1].evaluate(pos, o).getLong()));
                    break;
                case PType.TYPECODE_FLOAT:
                case PType.TYPECODE_OBJFLOAT:
                    result.setValue(evaluate(expressions[0].evaluate(pos, o).getFloat(),
									expressions[1].evaluate(pos, o).getFloat()));
                    break;
                case PType.TYPECODE_DOUBLE:
                case PType.TYPECODE_OBJDOUBLE:
                    result.setValue(evaluate(expressions[0].evaluate(pos, o).getDouble(),
									expressions[1].evaluate(pos, o).getDouble()));
                    break;
                case PType.TYPECODE_STRING:
                    result.setValue(evaluate(expressions[0].evaluate(pos, o).getString(),
									expressions[1].evaluate(pos, o).getString()));
                    break;
                case PType.TYPECODE_BIGDECIMAL:
                    result.setValue(evaluate(expressions[0].evaluate(pos, o).getBigDecimal(),
									expressions[1].evaluate(pos, o).getBigDecimal()));
                    break;
                case PType.TYPECODE_BIGINTEGER:
                    result.setValue(evaluate(expressions[0].evaluate(pos, o).getBigInteger(),
									expressions[1].evaluate(pos, o).getBigInteger()));
                    break;
                default:
				    // This case don't occurre if we compile at first
                    throw new TypingException("Incompatible type");
            }
        } catch (NullPointerException e) {
			// result == null or missing children objects
            throw new IllegalStateException("Unevaluable Expression: Not compiled");
        }
		return result;
    }

    // IMPLEMENTATION OF THE BinaryArithmeticOperator INTERFACE //
    //----------------------------------------------------------//
    public abstract int evaluate(int op1, int op2);

    public abstract int evaluate(int op1, short op2);

    public abstract long evaluate(int op1, long op2);

    public abstract float evaluate(int op1, float op2);

    public abstract double evaluate(int op1, double op2);

    public abstract float evaluate(float op1, float op2);

    public abstract float evaluate(float op1, short op2);

    public abstract float evaluate(float op1, int op2);

    public abstract float evaluate(float op1, long op2);

    public abstract double evaluate(float op1, double op2);

    public abstract int evaluate(char op1, char op2)
			throws TypingException;

    public abstract String evaluate(char op1, String op2)
            throws TypingException;

    public abstract long evaluate(long op1, long op2);

    public abstract long evaluate(long op1, short op2);

    public abstract long evaluate(long op1, int op2);

    public abstract float evaluate(long op1, float op2);

    public abstract double evaluate(long op1, double op2);

    public abstract double evaluate(double op1, double op2);

    public abstract double evaluate(double op1, short op2);

    public abstract double evaluate(double op1, int op2);

    public abstract double evaluate(double op1, float op2);

    public abstract double evaluate(double op1, long op2);

    public abstract BigDecimal evaluate(BigDecimal op1, BigDecimal op2);

    public abstract BigInteger evaluate(BigInteger op1, BigInteger op2);

    public abstract String evaluate(String op1, String op2)
            throws TypingException;

    public abstract String evaluate(String op1, char op2)
            throws TypingException;

    public abstract Date evaluate(Date op1, Date op2) throws TypingException;

    public Operand compileExpression()
		throws ExpressionException, MalformedExpressionException {
        compileOperands();
        if ((expressions[0] != null) && (expressions[1] != null)) {
            if (!(expressions[0].getType() == null || expressions[1].getType() == null)) {
                if ((ExpressionTypeHelper.isArithmeticType(expressions[0].getType()))
					&& (ExpressionTypeHelper.isArithmeticType(expressions[1].getType()))) {
                    if (expressions[0].getType().isa(expressions[1].getType())) {
                        type = expressions[1].getType();
                    } else {
                        type = expressions[0].getType();
                    }
                    result = new BasicVariableOperand(type);
                    verified = true;
                } else
                    throw new TypingException("Attempt an aritmetic type " + expressions[0].getType().getJavaName() + "  " + expressions[1].getType().getJavaName()); // it is not a numeric type
            } else
                throw new MalformedExpressionException("null children value");
        } else
		// Operands not yet assigned
            throw new MalformedExpressionException("null children value");
		return result;
    }
}
